<?php

if(!defined( 'ABSPATH'))
	die();


/**
 * This function displays the reCAPTCHA form using API keys supplied in the
 * admin panel.
 * @uses recaptcha_get_html()
 */
function lzr_display_recaptcha() {
	echo recaptcha_get_html( IvySetting::get( 'recaptcha_public_key' ) );
}


function lzr_get_url($c)
{
	return IvyController::get_permalink($c);
}


/**
 * This function gets the URL of the current visited page.
 *
 * @return string
 */
function lzr_get_current_url() {
	$pageURL = 'http';
	if (isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"] == "on") {
	   $pageURL .= "s";
	}
	$pageURL .= "://";
	if (isset($_SERVER['SERVER_PORT']) && $_SERVER["SERVER_PORT"] != "80") {
	     $pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"];
	} 
	else {
	     $pageURL .= $_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
	}
	return $pageURL;
}



/**
 * Display notices in template.
 */
function lzr_display_notices($notices, $form = null)
{
	if(isset($notices) && is_array($notices))
	{
		foreach($notices as $notice)
		{
			?>
			<div class="lzr-admin-notice <?php echo esc_attr($notice->type); ?>">
				<h5><?php echo $notice->title; ?></h5>
				<?php echo $notice->text; ?>
			</div>
			<?php
		}
	}
	
	if(isset($form) && !$form->is_valid())
	{
		?>
		<div id="lzr-errors">
			<h5>Something went wrong...</h5>
			<ul>
				<?php $form->display_errors(); ?>
			</ul>
		</div>
		<?php
	}
}


/**
 * Display  a form field in a  template.
 */
function lzr_display_form_field($key, $field, $value)
{
	if(!isset($field['type']))
		$field['type'] = 'text';
	
	if(!isset($field['class']))
		$field['class'] = array();
	else if(!is_array($field['class']))
		$field['class'] = array($field['class']);
	
	if(isset($field['description']))
		$descr = '<span class="description">'.esc_html($field['description']).'</span>';
	else
		$descr = '';
	
	// Dirty...
	if(isset($field['rules']) && is_array($field['rules']) && in_array('numeric', $field['rules']) && (int)$value != $value)
		$value = number_format_i18n($value, 2);
	
	switch($field['type'])
	{
		case 'hidden':
			?>
			<input type="hidden" id="lzr-<?php echo esc_attr($key); ?>" name="<?php echo esc_attr($key); ?>" value="<?php echo esc_attr($value); ?>">
			<?php
			break;
		case 'select':
			?>
			<div class="lzr-form-field lzr-form-field-<?php echo $field['type']; ?> <?php echo implode(' ', $field['class']); ?>">
				<label for="lzr-<?php echo esc_attr($key); ?>"><?php esc_html_e($field['title']); ?></label>
				<select id="lzr-<?php echo esc_attr($key); ?>" name="<?php echo esc_attr($key); ?>">
					<?php
					foreach($field['options'] as $val => $text)
					{
						?>
						<option value="<?php echo esc_attr($val); ?>" <?php selected($value, $val); ?>><?php esc_html_e($text); ?></option>
						<?php
					}
					?>
				</select>
				<?php echo $descr; ?>
			</div>
			<?php
			break;
		case 'checkboxes':
			?>
			<div class="lzr-form-field lzr-form-field-<?php echo $field['type']; ?> <?php echo implode(' ', $field['class']); ?>">
				<label><?php esc_html_e($field['title']); ?></label>
				<div class="checkboxes-wrap">
					<?php
					$i = 0;
					foreach($field['options'] as $val => $text)
					{
						?>
						<input type="checkbox" id="lzr-<?php echo esc_attr($key); ?>-<?php echo $i; ?>" name="<?php echo esc_attr($key); ?>[]" value="<?php echo esc_attr($val); ?>" <?php checked(in_array($val, $value)); ?>>
						<label for="lzr-<?php echo esc_attr($key); ?>-<?php echo $i; ?>" class="checkbox-label"><?php echo esc_html($text); ?></label><br>
						<?php
						$i++;
					}
					?>
					<?php echo $descr; ?>
				</div>
			</div>
			<?php
			break;
		case 'checkbox':
			?>
			<div class="lzr-form-field lzr-form-field-<?php echo $field['type']; ?> <?php echo implode(' ', $field['class']); ?>">
				<label for="lzr-<?php echo esc_attr($key); ?>"><?php esc_html_e($field['title']); ?></label>
				<input type="checkbox" id="lzr-<?php echo esc_attr($key); ?>" name="<?php echo esc_attr($key); ?>" <?php checked($value); ?>>
				<?php echo $descr; ?>
			</div>
			<?php
			break;
		case 'textarea':
			?>
			<div class="lzr-form-field lzr-form-field-<?php echo $field['type']; ?> <?php echo implode(' ', $field['class']); ?>">
				<label for="lzr-<?php echo esc_attr($key); ?>"><?php esc_html_e($field['title']); ?></label>
				<textarea rows="12" id="lzr-<?php echo esc_attr($key); ?>" name="<?php echo esc_attr($key); ?>"><?php echo esc_textarea($value); ?></textarea>
				<?php echo $descr; ?>
			</div>
			<?php
			break;
		case 'password':
			?>
			<div class="lzr-form-field lzr-form-field-<?php echo $field['type']; ?> <?php echo implode(' ', $field['class']); ?>">
				<label for="lzr-<?php echo esc_attr($key); ?>"><?php esc_html_e($field['title']); ?></label>
				<input type="password" autocomplete="off" id="lzr-<?php echo esc_attr($key); ?>" name="<?php echo esc_attr($key); ?>" value="<?php echo esc_attr($value); ?>">
				<?php echo $descr; ?>
			</div>
			<?php
			break;
		case 'custom':
			break;
		case 'text':
		default:
			?>
			<div class="lzr-form-field lzr-form-field-<?php echo $field['type']; ?> <?php echo implode(' ', $field['class']); ?>">
				<label for="lzr-<?php echo esc_attr($key); ?>"><?php esc_html_e($field['title']); ?></label>
				<input type="text" id="lzr-<?php echo esc_attr($key); ?>" name="<?php echo esc_attr($key); ?>" value="<?php echo esc_attr($value); ?>">
				<?php echo $descr; ?>
			</div>
			<?php
			break;
	}
}


function lzr_month_fields($month)
{
	for($i = 1; $i <= 12; $i++)
	{
		$m = date_i18n('F', mktime(0, 0, 0, $i, 1, 2012));
		?>
		<option value="<?php echo $i; ?>" <?php selected($i, $month); ?>><?php echo ucfirst($m); ?></option>
		<?php
	}
}


function lzr_year_fields($year, $start = null, $end = null)
{
	if(!isset($start))
		$start = (int)date('Y') - 3;
	
	if(!isset($end))
		$end = (int)date('Y') + 3;
	
	for($i = $start; $i <= $end; $i++)
	{
		?>
		<option value="<?php echo $i; ?>" <?php selected($i, $year); ?>><?php echo $i; ?></option>
		<?php
	}
}


function floatval_i18n($str)
{
	global $wp_locale;
	
	$str = (string)$str;
		
	$str = str_replace($wp_locale->number_format['thousands_sep'], '', $str);
	$str = str_replace($wp_locale->number_format['decimal_point'], '.', $str);
	
	return floatval($str);
}


function is_numeric_i18n($str)
{
	global $wp_locale;
	
	$str = (string)$str;
		
	$str = str_replace($wp_locale->number_format['thousands_sep'], '', $str);
	$str = str_replace($wp_locale->number_format['decimal_point'], '.', $str);
	
	return is_numeric($str);
}


?>