<?php 
if ( ! defined( 'ABSPATH' ) )
	die( "Can't load this file directly" );

class IvyController
{
	private static $active_frontend_controller = '';
	private static $active_frontend_controller_method = '';
	private static $active_frontend_controller_arguments = array();
	private static $buffer;
	private static $controllers = array();
	private static $shortcode_map = array();
	protected static $identifier = '';
	private static $loaded_controllers = array();
	
	public function __construct() {
		add_filter( 'body_class', array( $this, 'filter_body_class' ) );
	}
	
	public function filter_body_class( $classes ) {
		$main_class = 'leezer-' . self::$active_frontend_controller;
		$classes[] = $main_class;
		$classes[] = 'leezer-controller';
		
		if ( self::$active_frontend_controller_method != 'index' )
			$classes[] = $main_class . '-' . self::$active_frontend_controller_method;
			
		return $classes;
	}
	
	public static function call_frontend_controller_method( $uri ) {
		if ( isset( $uri[0] ) )
			$uri[0] = str_replace( array( '-', ' ' ), '_', $uri[0] );
		if ( count( $uri ) > 0 && strpos( $uri[0], 'action' ) === false && strpos( $uri[0], 'ajax' ) === false && method_exists( self::$loaded_controllers[self::$active_frontend_controller], $uri[0] ) )
			self::$active_frontend_controller_method = array_shift( $uri );
		else if(defined('DOING_AJAX') && DOING_AJAX)
			self::$active_frontend_controller_method = 'ajax';
		else
			self::$active_frontend_controller_method = 'index';

		self::$active_frontend_controller_arguments = $uri;
		ob_start();
		call_user_func_array( array( self::$loaded_controllers[self::$active_frontend_controller], self::$active_frontend_controller_method ), self::$active_frontend_controller_arguments );
		self::$buffer = ob_get_clean();
	}
	
	public static function frontend_controller_output() {
		return self::$buffer;
	}
	
	public static function generate_rewrite_rules( $rules ) {
		$new_rules = array();
		foreach ( self::$shortcode_map as $controller => $page ) {
			$new_rules[ "{$page['slug']}((/[^/]*)*)/?$" ] = 'index.php?pagename=' . $page['slug'] . '&' . self::$identifier .  '_query=' . $controller . '$matches[1]';
		}
		$rules = array_merge( $new_rules, $rules );
		return $rules;
	}
	
	public static function get_permalink( $controller ) {
		if ( isset( self::$shortcode_map[$controller] ) ) {
			return self::$shortcode_map[$controller]['permalink'];
		}
		return false;
	}
	
	public static function get_permalink_by_page_slug( $slug ) {
		foreach ( self::$shortcode_map as $controller => $page ) {
			if ( $page['slug'] == $slug )
				return $page['permalink'];
		}
		
		return false;
	}
	
	public static function is( $controller ) {
		return in_array( $controller, array_keys( self::$loaded_controllers ) );
	}
	
	private static function load_controller( $controller_name, $type ) {
		require_once( IvyCore::get_path() . 'controllers/' . $type . '/' . $controller_name . '.php' );
		$sanitized_controller_name = str_replace( array( '-', '_' ), ' ', $controller_name );
		$sanitized_controller_name = str_replace( ' ', '', ucwords( $sanitized_controller_name ) );
		$classname = strtoupper( self::$identifier ) . '_' . $sanitized_controller_name . 'Controller';
		self::$loaded_controllers[$controller_name] = new $classname;
	}
	
	private static function load_controller_map() {
		self::$shortcode_map = IvySetting::get( 'shortcode_map' );
	}
	
	public static function load_frontend_controller( $controller_name ) {
		do_action( self::$identifier . '_init_' . $controller_name );
		do_action( self::$identifier . '_init_controller' );
		self::load_controller( $controller_name, 'frontend' );
		self::$active_frontend_controller = $controller_name; 
		do_action( self::$identifier . '_loaded_' . $controller_name );
		do_action( self::$identifier . '_loaded_controller' );
	}
	
	public static function register( $controllers ) {
		self::$controllers = $controllers;
	}
	
	public static function set_identifier( $identifier ) {
		self::$identifier = $identifier;
	}
	
	public static function setup_frontend_controllers() {
		if ( ! is_admin() || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
			self::load_controller_map();
			foreach ( self::$shortcode_map as $controller => $page ) {
				$shortcode = self::$identifier . '-' . $controller;
				add_shortcode( $shortcode, array( 'IvyController', 'frontend_controller_output' ) );
			}
		}
	}
	
	public static function map_to_shortcodes() {
		global $wpdb;
		
		$map = array();
		
		foreach ( self::$controllers as $controller ) {
			$shortcode = self::$identifier . '-' . $controller;

			$like = like_escape( "[{$shortcode}" );
			$query = "SELECT ID, post_name FROM " . $wpdb->posts . " WHERE post_status = 'publish' AND post_type = 'page' AND post_content LIKE '%{$like}%'";
			$page = $wpdb->get_row( $query );
			
			if ( $page ) 
				$map[$controller] = array(
					'id'         => $page->ID,
					'slug'       => $page->post_name,
					'permalink'  => get_permalink( $page->ID ),
					);
		}
		
		IvySetting::set( 'shortcode_map', $map );
		self::$shortcode_map = $map;
	}
}