<?php 
if ( ! defined( 'ABSPATH' ) )
	die( "Can't load this file directly" );

require_once( 'includes/lang-wrapper.php' );

require_once( 'classes/controller.php' );
require_once( 'classes/view.php' );
require_once( 'classes/setting.php' );

if ( get_magic_quotes_gpc() ) {
    $_POST      = array_map( 'stripslashes_deep', $_POST );
    $_GET       = array_map( 'stripslashes_deep', $_GET );
    $_COOKIE    = array_map( 'stripslashes_deep', $_COOKIE );
    $_REQUEST   = array_map( 'stripslashes_deep', $_REQUEST );
}

class IvyCore
{
	private static $path      = '';
	private static $core_path = '';
	private static $url       = '';
	private static $css_url   = '';
	private static $img_url   = '';
	private static $js_url    = '';
	private static $identifier;
	private static $db_prefix = '';
	protected $all_caps = array();
	protected static $settings = array();
	
	public function __construct( $plugin_file, $identifier ) {
		self::$identifier = $identifier;
		
		global $wpdb;
		self::$db_prefix = $wpdb->prefix . $identifier . '_';
		
		add_action( 'init', array( $this, 'action_register_post_types' ) );
		
		add_action( 'save_post'         , array( $this, 'action_save_post'     ), 10, 2 );
		add_action( 'parse_request'     , array( $this, 'action_parse_request' ) );
		
		add_action('wp_ajax_lzr_get', array($this, 'action_ajax'));
		
		add_filter( 'query_vars'        , array( $this, 'filter_query_vars'    ) );
		add_filter( 'page_rewrite_rules', array( $this, 'filter_rewrite_rules' ) );
		add_filter( 'map_meta_cap'      , array( $this, 'filter_map_meta_cap'  ), 10, 4 );

		register_activation_hook( $plugin_file, array( $this, 'action_activate' ) );

		// set the identifiers for necessary classes
		IvySetting::set_identifier( self::$identifier );
		IvyController::set_identifier( self::$identifier );
		IvyController::setup_frontend_controllers();
		IvyController::register( $this->shortcode_controllers );
		
		self::construct_paths_and_urls( $plugin_file );

		load_plugin_textdomain( self::$identifier . '_lang', false, basename(dirname(dirname(__FILE__))).'/languages/');
	}
	
	private static function construct_paths_and_urls( $file ) {
		self::$path      = trailingslashit( plugin_dir_path( $file ) );
		self::$core_path = trailingslashit( dirname( __FILE__ ) );
		self::$url       = trailingslashit( plugin_dir_url( $file ) );
		self::$css_url   = self::$url . 'css/';
		self::$img_url   = self::$url . 'img/';
		self::$js_url    = self::$url . 'js/';
	}
	
	public function action_activate() {
		if ( ! empty( self::$settings ) ) {
			foreach ( self::$settings as $name => $default_value ) {
				if ( $name[0] == '*' ) {
					$autoload = 'yes';	
					$name = str_replace( '*', '', $name );
				}
				else
				{
					$autoload = 'no';
				}
				
				IvySetting::add( $name, $default_value, $autoload );
			}
		}
		
		$this->action_register_post_types();
		$this->register_post_type_caps();
		IvyController::map_to_shortcodes();
		IvySetting::set( 'first_run', 'yes' );
	}
	
	public function action_parse_request( $query ) {
		global $wp_rewrite;

		$var_name = self::$identifier . '_query';

		if ( isset( $query->query_vars[$var_name] ) ) {
			$request          = $query->query_vars[$var_name];
			$permalink        = IvyController::get_permalink_by_page_slug( $query->query_vars['pagename'] );

			$components       = explode( '/', $request );
			$controller_name  = array_shift( $components );
			
			IvyController::load_frontend_controller( $controller_name );			
			IvyController::call_frontend_controller_method( $components );
		}
	}
	
	
	public function action_ajax()
	{
		if(isset($_GET['c']) && in_array($_GET['c'], $this->shortcode_controllers))
		{
			IvyController::load_frontend_controller($_GET['c']);
			IvyController::call_frontend_controller_method(array());
			
			exit(IvyController::frontend_controller_output());
		}
	}
	
	public function action_register_post_types() {

		if ( ! empty( $this->taxonomies ) ) {
			foreach ( $this->taxonomies as $post_type => $taxonomies ) {
				foreach ( $taxonomies as $name => $args ) {
					register_taxonomy( $name, $post_type, $args );
				}
			}
		}
		
		if ( ! empty( $this->post_types ) ) {
			// register post types
			foreach ( $this->post_types as $post_type => $args ) {
				register_post_type( $post_type, $args );
			}	
		}
		
		if ( IvySetting::get( 'first_run' ) == 'yes' ) {
			flush_rewrite_rules( false );
			IvySetting::set( 'first_run', 'no' );
		}
	}
	
	public function action_save_post( $post_ID, $post ) {
		global $wp_rewrite;
		
		if ( $post->post_type == 'page' && $post->post_status == 'publish' && strpos( $post->post_content, '[' . self::$identifier . '-' ) !== false && ! $wp_rewrite->using_permalinks() ) {
			IvyController::map_to_shortcodes();
			flush_rewrite_rules( false );
		}
	}
	
	public function filter_map_meta_cap( $caps, $cap, $user_id, $args ) {
		$meta_cap = array_search( $cap, $this->all_caps );

		if ( ! empty( $meta_cap ) && in_array( $meta_cap, array( 'edit_post', 'read_post', 'delete_post' ) ) ) {
			$post      = get_post( $args[0] );
			$post_type = get_post_type_object( 'community_link' );
			$caps      = array();

			switch ( $meta_cap ) {
				case 'edit_post':
					if ( is_null( $post ) || $user_id == $post->post_author )
						$caps[] = $post_type->cap->edit_post;
					else
						$caps[] = $post_type->cap->edit_others_posts;
					break;
				
				case 'read_post':
					if ( 'private' != $post->post_status )
						$caps[] = 'read';
					elseif ( $user_id == $post->post_author )
						$caps[] = 'read';
					else
						$caps[] = $post_type->cap->read_private_posts;
					break;
				
				case 'delete_post':
					$user = new WP_User( $user_id );

					if ( $user->has_cap( 'delete_posts' ) || $user_id == $post->post_author )
						$caps[] = $post_type->cap->edit_posts;						
					else
						$caps[] = $post_type->cap->edit_others_posts;
					break;
			}
		}

		return $caps;
	}
	
	public function filter_query_vars( $vars ) {
		$vars[] = self::$identifier . '_query';

		return $vars;
	}
	
	public function filter_rewrite_rules( $rules ) {
		IvyController::map_to_shortcodes();
		$rules =  IvyController::generate_rewrite_rules( $rules );
		return $rules;
	}
	
	public static function get_db_prefix() {
		return self::$db_prefix;
	}
	
	public static function get_identifier() {
		return self::$identifier;
	}
	
	public static function get_textdomain() {
		return self::$identifier . '_lang';
	}
	
	public static function get_path( $type = '' ) {
		switch ( $type ) {
			case 'css':
				return self::$css_path;
				break;
			
			default:
				return self::$path;
				break;
		}
	}
	
	public static function get_url( $type = '' ) {
		switch ( $type ) {
			case 'css':
				return self::$css_url;
				break;
				
			case 'img':
				return self::$img_url;
				break;
			
			case 'js':
				return self::$js_url;
				break;
			
			default:
				return self::$url;
				break;
		}
	}
	
	public static function load_core_library( $library ) {
		require_once( self::$core_path . 'classes/' . $library . '.php' );
		$sanitized_library_name = str_replace( array( '-', '_' ), ' ', $library );
		$sanitized_library_name = str_replace( ' ', '', ucwords( $sanitized_library_name ) );
		$classname = 'Ivy' . $sanitized_library_name;
		
		if ( method_exists( $classname, 'set_identifier' ) ) {
			call_user_func_array( array( $classname, 'set_identifier' ), array( self::$identifier ) );
		}
	}
	
	public static function load_library( $library ) {
		require_once( self::$path . 'libraries/' . $library . '.php' );
		$sanitized_library_name = str_replace( array( '-', '_' ), ' ', $library );
		$sanitized_library_name = str_replace( ' ', '', ucwords( $sanitized_library_name ) );
		$classname = strtoupper( self::$identifier ) . $sanitized_library_name;
		
		if ( method_exists( $classname, 'set_identifier' ) ) {
			call_user_func_array( array( $classname, 'set_identifier' ), array( self::$identifier ) );
		}
	}
	
	public function register_post_type_caps() {
		$caps_schema = IvySetting::get( 'caps_schema' );
		
		if ( ! $caps_schema )
			$caps_schema = array();
			
		$post_types_caps = array_merge( $this->post_types_caps, $caps_schema );

		foreach ( $post_types_caps as $post_type => $type_schema ) {
			foreach ( $type_schema as $role_name => $caps ) {
				$role =& get_role( $role_name );
				
				foreach ( $this->all_caps as $post_cap => $cap ) {

					if ( in_array( $cap, $caps ) && ! $role->has_cap( $cap ) ) {
						$role->add_cap( $cap );
					}

					if ( $role->has_cap( $cap ) && ! in_array( $cap, $caps ) ) {

						$role->remove_cap( $cap );
					}
				}
			}
		}
		
		if ( empty( $caps_schema ) )
			IvySetting::set( 'caps_schema', $post_types_caps );
	}
		
}

if ( ! function_exists( 'wp_new_user_notification' ) ) {
	function wp_new_user_notification($user_id, $plaintext_pass = '') {
		$user = new WP_User($user_id);

		$user_login = stripslashes($user->user_login);
		$user_email = stripslashes($user->user_email);

		// The blogname option is escaped with esc_html on the way into the database in sanitize_option
		// we want to reverse this for the plain text arena of emails.
		$blogname = wp_specialchars_decode(get_option('blogname'), ENT_QUOTES);

		$message  = sprintf(__('New user registration on your site %s:'), $blogname) . "\r\n\r\n";
		$message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
		$message .= sprintf(__('E-mail: %s'), $user_email) . "\r\n";
		
		$message = apply_filters( IvyCore::get_identifier() . '_registration_report_email_content', $message );

		@wp_mail(get_option('admin_email'), sprintf(__('[%s] New User Registration'), $blogname), $message);

		if ( empty($plaintext_pass) )
			return;

		$headers = "From: Artikel Marketing <".get_bloginfo('admin_email').">\r\n";
		$headers .= "Reply-to: Artikel Marketing <".get_bloginfo('admin_email').">";
	
		$message  = sprintf( ivy__( 'Thank you for registering at %s. Below you can find your username and password. You\'re advised to change this password after you log in for the first time.' ), $blogname ) . "\r\n\r\n";
		$message .= sprintf(__('Username: %s'), $user_login) . "\r\n";
		$message .= sprintf(__('Password: %s'), $plaintext_pass) . "\r\n";
		$message .= IvyController::get_permalink( 'login' ) . "\r\n";

		$message = apply_filters( IvyCore::get_identifier() . '_registration_email_content', $message );

		wp_mail($user_email, 'Uw gebruikersnaam en wachtwoord', $message, $headers);

	}
}